package org.iphsoft.simon1;

import android.app.Activity;
import android.content.Intent;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.view.View;
import android.view.WindowManager;
import android.view.animation.Animation;
import android.view.animation.Animation.AnimationListener;
import android.view.animation.AnimationUtils;

import com.R;

import org.iphsoft.simon1.AndroidPortAdditions.GameType;
import org.iphsoft.simon1.stats.StatisticsManager;
import org.iphsoft.simon1.ui.ActivityAnimationUtil;
import org.iphsoft.simon1.util.BackgroundMusicPlayer;
import org.iphsoft.simon1.util.BitmapUtils;
import org.iphsoft.simon1.util.ImmersiveModeHelper;
import org.iphsoft.simon1.util.MyLog;
import org.iphsoft.simon1.util.TestGLActivity;

public class SplashActivity extends Activity {

	@Override
	protected void onCreate(Bundle savedInstanceState)
	{

		super.onCreate(savedInstanceState);

		// Set hardware acceleration to make animations fluid
		if (Build.VERSION.SDK_INT >= 11)
		{
			MyLog.d("SplashActivity: onCreate: setting hardware accelerated");
			getWindow().setFlags(WindowManager.LayoutParams.FLAG_HARDWARE_ACCELERATED,
					WindowManager.LayoutParams.FLAG_HARDWARE_ACCELERATED);
		}

		setContentView(R.layout.splash);

		mImmersiveModeHelper = new ImmersiveModeHelper(this);

		View bgFrame = findViewById(R.id.bg_frame);
		BitmapUtils.setScaledBackground(this, bgFrame, R.drawable.splash);

		if (AndroidPortAdditions.GAME_TYPE != GameType.IHNM)
		{
			// Start a background check for ad policy
			AdHelper.startBackgroundAdCheck();
		}

		// Start the splash animation after a delay
		mHandler = new Handler();
		switch (AndroidPortAdditions.GAME_TYPE)
		{
		case SIMON1:
			setupSimon1Animation();
			break;
		case SIMON2:
			setupSimon2Animation();
			break;
		case T7G:
		case FOTAQ:
			setupT7gAnimation();
			break;
		case IHNM:
			setupIHNMAnimation();
			break;
		}

		if (AndroidPortAdditions.instance().getFirstUse())
		{
			// If this is the first run, launch an activity to query the
			// GL_RENDERER string
			startActivityForResult(new Intent(this, TestGLActivity.class), GL_TEST_REQUEST_CODE);
		} else
		{
			// If this is not the first run, queue the runnable normally
			mHandler.postDelayed(mAnimateRunnable, DELAY);
		}
	}

	@Override
	protected void onDestroy()
	{

		// Remove all pending callbacks (in case user pressed back and next
		// activity wasn't launched yet)
		mHandler.removeCallbacks(mAnimateRunnable);
		super.onDestroy();
	}

	@Override
	protected void onStart()
	{
		super.onStart();

		StatisticsManager.getStatistics().reportActivityStart(this);

		if (AndroidPortAdditions.HAS_BACKGROUND_MUSIC)
		{
			BackgroundMusicPlayer.addBackgroundMusicActivity();
		}
	}

	@Override
	protected void onStop()
	{
		super.onStop();

		//StatisticsManager.getStatistics().reportActivityStop(this);

		if (AndroidPortAdditions.HAS_BACKGROUND_MUSIC)
		{
			BackgroundMusicPlayer.removeBackgroundMusicActivity();
		}
	}

	@Override
	protected void onResume()
	{
		// TODO Auto-generated method stub
		super.onResume();

		mImmersiveModeHelper.supportFullScreenImmersiveMode();
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data)
	{

		// Check for the GL_RENDERER result
		if (requestCode == GL_TEST_REQUEST_CODE && resultCode == RESULT_OK && data != null)
		{
			String glRenderer = data.getStringExtra(TestGLActivity.EXTRA_GL_RENDERER);
			if (glRenderer != null)
			{
				AndroidPortAdditions.instance().setGlRendererPref(glRenderer);
			}
		}

		// Queue the splash animation
		mHandler.postDelayed(mAnimateRunnable, DELAY);

		super.onActivityResult(requestCode, resultCode, data);
	}

	private void nextActivity()
	{
		startActivity(new Intent(this, GameMenuActivity.class));
		finish();
		ActivityAnimationUtil.makeSplashActivityFadeTransition(this);
	}

	private void setupSimon1Animation()
	{
		mAnimateRunnable = new Runnable()
		{

			@Override
			public void run()
			{
				// Start the animation, and switch to the next activity when it
				// ends
				View insideView = findViewById(R.id.splashCenterLayout);

				Animation fadeoutAnim = AnimationUtils.loadAnimation(SplashActivity.this,
						R.anim.simon1_splash_inside_fade_out);

				fadeoutAnim.setAnimationListener(new AnimationListener()
				{

					@Override
					public void onAnimationStart(Animation arg0)
					{
					}

					@Override
					public void onAnimationRepeat(Animation arg0)
					{
					}

					@Override
					public void onAnimationEnd(Animation arg0)
					{
						nextActivity();
					}
				});

				insideView.startAnimation(fadeoutAnim);
			}
		};
	}

	private void setupSimon2Animation()
	{
		if (Build.VERSION.SDK_INT >= 14)
		{
			mAnimateRunnable = new Runnable()
			{

				@Override
				public void run()
				{

					// Start the animation, and switch to the next activity when
					// it
					// ends
					Animation leftDoorAnim = AnimationUtils.loadAnimation(SplashActivity.this,
							R.anim.simon2_splash_left_door);
					Animation rightDoorAnim = AnimationUtils.loadAnimation(SplashActivity.this,
							R.anim.simon2_splash_right_door);
					final Animation logoAnim = AnimationUtils.loadAnimation(SplashActivity.this,
							R.anim.simon2_splash_upper_logo);
					final Animation subtitleAnim = AnimationUtils.loadAnimation(
							SplashActivity.this, R.anim.simon2_splash_subtitle);

					View leftDoor = findViewById(R.id.img_door_left);
					BitmapUtils.setScaledBackground(SplashActivity.this, leftDoor,
							R.drawable.bg_door_left);
					View rightDoor = findViewById(R.id.img_door_right);
					BitmapUtils.setScaledBackground(SplashActivity.this, rightDoor,
							R.drawable.bg_door_right);

					final View logo = findViewById(R.id.img_logo);
					final View subtitle = findViewById(R.id.subtitle_view);

					leftDoorAnim.setAnimationListener(new AnimationListener()
					{
						@Override
						public void onAnimationStart(Animation animation)
						{
						}

						@Override
						public void onAnimationRepeat(Animation animation)
						{
						}

						@Override
						public void onAnimationEnd(Animation animation)
						{
							logo.startAnimation(logoAnim);
						}
					});

					logoAnim.setAnimationListener(new AnimationListener()
					{

						@Override
						public void onAnimationStart(Animation animation)
						{
						}

						@Override
						public void onAnimationRepeat(Animation animation)
						{
						}

						@Override
						public void onAnimationEnd(Animation animation)
						{
							subtitle.startAnimation(subtitleAnim);
						}
					});

					subtitleAnim.setAnimationListener(new AnimationListener()
					{

						@Override
						public void onAnimationStart(Animation arg0)
						{
						}

						@Override
						public void onAnimationRepeat(Animation arg0)
						{
						}

						@Override
						public void onAnimationEnd(Animation arg0)
						{
							nextActivity();
						}
					});

					leftDoor.startAnimation(leftDoorAnim);
					rightDoor.startAnimation(rightDoorAnim);
				}
			};
		} else
		{
			// Hack for Galaxy S1 animation bug, since we don't know yet how to
			// solve.
			// Just don't do the splash sequence on 2.3 devices.
			mAnimateRunnable = new Runnable()
			{

				@Override
				public void run()
				{
					nextActivity();
				}
			};

			View leftDoor = findViewById(R.id.img_door_left);
			BitmapUtils.setScaledBackground(SplashActivity.this, leftDoor, R.drawable.bg_door_left);
			View rightDoor = findViewById(R.id.img_door_right);
			BitmapUtils.setScaledBackground(SplashActivity.this, rightDoor,
					R.drawable.bg_door_right);

			final View logo = findViewById(R.id.img_logo);
			final View subtitle = findViewById(R.id.subtitle_view);

			leftDoor.setVisibility(View.VISIBLE);
			rightDoor.setVisibility(View.VISIBLE);
			logo.setVisibility(View.VISIBLE);
			subtitle.setVisibility(View.VISIBLE);
		}
	}

	private void setupIHNMAnimation()
	{

		mAnimateRunnable = new Runnable()
		{

			@Override
			public void run()
			{
				nextActivity();
			}
		};

		BitmapUtils.setScaledBackground(this, findViewById(R.id.bg_frame), R.drawable.bg);
	}

	private void setupT7gAnimation()
	{
		final View board = findViewById(R.id.bg2);
		final View logo = findViewById(R.id.img_logo);
		final View sphinx = findViewById(R.id.img_sphinx);
		final View subtitle = findViewById(R.id.subtitle_view);

		board.setVisibility(View.INVISIBLE);
		logo.setVisibility(View.INVISIBLE);
		// sphinx.setVisibility(View.INVISIBLE);
		subtitle.setVisibility(View.INVISIBLE);

		BitmapUtils.setScaledBackground(this, board, R.drawable.bg);

		// Start the animation, and switch to the next activity when it
		// ends
		final Animation boardAnim = AnimationUtils.loadAnimation(SplashActivity.this,
				R.anim.t7g_splash_board);
		final Animation fadeInAnim1 = AnimationUtils.loadAnimation(SplashActivity.this,
				R.anim.t7g_splash_fade_in);
		final Animation fadeInAnim2 = AnimationUtils.loadAnimation(SplashActivity.this,
				R.anim.t7g_splash_fade_in);
		final Animation subtitleAnim = AnimationUtils.loadAnimation(SplashActivity.this,
				R.anim.t7g_splash_20th_anniversary);

		boardAnim.setAnimationListener(new AnimationListener()
		{

			@Override
			public void onAnimationStart(Animation animation)
			{
			}

			@Override
			public void onAnimationRepeat(Animation animation)
			{
			}

			@Override
			public void onAnimationEnd(Animation animation)
			{
				// sphinx.startAnimation(fadeInAnim1);
				logo.startAnimation(fadeInAnim2);
			}
		});

		fadeInAnim2.setAnimationListener(new AnimationListener()
		{

			@Override
			public void onAnimationStart(Animation animation)
			{
			}

			@Override
			public void onAnimationRepeat(Animation animation)
			{
			}

			@Override
			public void onAnimationEnd(Animation animation)
			{
				subtitle.startAnimation(subtitleAnim);
			}
		});

		subtitleAnim.setAnimationListener(new AnimationListener()
		{

			@Override
			public void onAnimationStart(Animation animation)
			{
			}

			@Override
			public void onAnimationRepeat(Animation animation)
			{
			}

			@Override
			public void onAnimationEnd(Animation animation)
			{
				nextActivity();
			}
		});

		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB)
		{
			mAnimateRunnable = new Runnable()
			{

				@Override
				public void run()
				{
					board.startAnimation(boardAnim);
				}
			};
		} else
		{
			// Hack for Galaxy S1 animation bug, since we don't know yet how to
			// solve.
			// Just don't do the splash sequence on 2.3 devices.
			mAnimateRunnable = new Runnable()
			{

				@Override
				public void run()
				{
					nextActivity();
				}
			};
		}
	}

	private Handler mHandler;
	private Runnable mAnimateRunnable;

	private ImmersiveModeHelper mImmersiveModeHelper;

	private static int DELAY;
	static
	{
		switch (AndroidPortAdditions.GAME_TYPE)
		{
		case SIMON1:
			DELAY = 1500;
			break;
		case SIMON2:
			DELAY = 500;
			break;
		case FOTAQ:
		case T7G:
			DELAY = 500;
			break;

		case IHNM:
			DELAY = 0;

			break;
		}
	}

	private static final int GL_TEST_REQUEST_CODE = 1;
}
